from si_tests import logger
from si_tests import settings
from si_tests.utils import waiters
from si_tests.managers.openstack_client_manager import OpenStackClientManager


LOG = logger.logger
test_params = {
    "server_name": "tls_vnc_test",
    "server_flavor": "m1.extra_tiny_test",
    "oscloud_user": "test",
}


def create_test_VM(ocm_test, params):
    LOG.info("2.1: Boot VM on compute host")
    ocm_test.server.create(
        [
            f"--image={settings.OPENSTACK_CIRROS_IMAGE_NAME}",
            "--flavor", params["server_flavor"],
            params["server_name"],
        ]
    )

    # Check VM is running
    LOG.info("2.2: Check VM is running")
    waiters.wait(
        lambda: ocm_test.server.show([params["server_name"]])[
            "OS-EXT-STS:power_state"
        ]
        in ["Running", 1],
        timeout=60,
        interval=10,
    )


def is_tls_active(ocm_test, domain):
    instance = ocm_test.server.show([domain])
    hostname = instance["OS-EXT-SRV-ATTR:host"]
    cmd = [
        '/bin/bash', '-c',
        f'ps axwwwocommand | grep guest={instance["OS-EXT-SRV-ATTR:instance_name"]} | grep -v grep'
    ]
    qemu_line = ocm_test.exec_libvirt(hostname, cmd)
    return "tls-creds-x509" in qemu_line and (
        "verify-peer=yes" in qemu_line or "verify-peer=on" in qemu_line
    )


def test_tls_vnc(os_manager, openstack_client_manager):
    """Verify activation TLS for VNC

    Scenario:
        1. Check OSDPL settings
        2. Create test VM
        3. Check TLS for VNC
        4. Clean up

    """
    LOG.info("Step 1: Check if TLS for console is enabled.")
    osdpl = os_manager.get_osdpl_deployment()
    assert (
        osdpl.data.get("spec")
        .get("features", {})
        .get("nova", {})
        .get("console", {})
        .get("novnc", {})
        .get("tls", {})
        .get("enabled", False)
    ), "TLS for VNC is not enabled in OSDPL."
    LOG.info("OSDPL settings Ok. TLS for VNC is enabled.")

    ocm_test = OpenStackClientManager(os_cloud=test_params["oscloud_user"])
    try:
        LOG.info("Step 2: Create test VM.")
        create_test_VM(ocm_test, test_params)

        LOG.info("Step 3: Check TLS for VNC")
        assert is_tls_active(
            ocm_test, test_params["server_name"]
         ), "No TLS for VNC in VM params"
    finally:
        LOG.info("Step 4: Cleanup")
        ocm_test.server.delete([test_params["server_name"]])
