#    Copyright 2022 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from urllib import parse as urlparse

from si_tests import logger
from si_tests import settings
from si_tests.managers import bootstrap_manager
from si_tests.utils import utils
from si_tests.utils.certs import CertManager

LOG = logger.logger


def test_set_cache_custom_cert(kaas_manager, show_step):
    """Add custom cache cert to the regional cluster.

    Scenario:
        1. Collecting env data
        2. Get seed ip and ssh-Key
        3. Apply mcc-cache TLS settings
        4. Validate mcc-cache connection on child clusters

    """

    show_step(1)
    cluster_name = settings.TARGET_CLUSTER
    namespace_name = settings.TARGET_NAMESPACE
    ns = kaas_manager.get_namespace(namespace_name)
    cluster = ns.get_cluster(cluster_name)

    show_step(2)
    seed_ip = kaas_manager.si_config.data.get('run_on_remote', {}).get('SEED_STANDALONE_EXTERNAL_IP')
    assert seed_ip, 'Seed node IP address not found'
    LOG.info(f"Seed node IP address {seed_ip}")
    key = kaas_manager.si_config.data['run_on_remote']['SEED_SSH_PRIV_KEY']
    with open(
            'key-file',
            mode='w') as f:
        f.write(key)
    bootstrap = bootstrap_manager.BootstrapManager(seed_ip=seed_ip, priv_key_file='key-file')

    host = urlparse.urlsplit(cluster.mcc_cache).hostname
    ip = None
    dns_name = None
    if utils.is_valid_ip(host):
        ip = host
    else:
        dns_name = host

    show_step(3)
    app = "cache"
    ca_path = app
    cert_dir = cluster.name

    cert_pem, key_pem, ca_pem = bootstrap.generate_cert(ip=ip, dns_name=dns_name, ca_path=ca_path, cert_dir=cert_dir)
    cert_manager = CertManager(kaas_manager=kaas_manager)
    cert_manager.apply_cert_for_app(app=app, cluster=cluster, hostname=host,
                                    cert_pem=cert_pem, key_pem=key_pem, ca_pem=ca_pem)

    cluster.check.check_cluster_readiness()

    show_step(4)
    child_clusters = kaas_manager.get_child_clusters_in_region(cluster.region_name)
    for child_cluster in child_clusters:
        child_cluster.check.validate_machines_mcc_ca(ca=ca_pem)
        child_cluster.check.check_cluster_readiness()

    cluster.store_k8s_artifacts()
